import { AuthorPayoutArgs, PayoutBatchItemCompositeKeySerialized } from '../types';
import { payoutQueueQueryParser, payoutQueueResultParser, payoutStatusResultParser } from '../parsers';
import z from 'zod';
import { UserId } from '../../user/types';
import {
  PAYOUT_QUEUE_ITEM_ATTEMPT_ENDED_AT_FIELD_NAME,
  PAYOUT_QUEUE_ITEM_ATTEMPT_ERRORS_FIELD_NAME,
  PAYOUT_QUEUE_ITEM_ATTEMPT_ID_PREFIX,
  PAYOUT_QUEUE_ITEM_ATTEMPT_STARTED_AT_FIELD_NAME,
  PAYOUT_QUEUE_ITEM_ATTEMPTS_FIELD_NAME,
} from '../constants';

export enum PaypalPayoutBatchIdBrand {
  _ = '',
}
export type PaypalPayoutBatchId = string & PaypalPayoutBatchIdBrand; // not equivalent to "sender batch id" which is our batch key

export type PayoutQueueItemAttempt = {
  [PAYOUT_QUEUE_ITEM_ATTEMPT_STARTED_AT_FIELD_NAME]: Date;
  [PAYOUT_QUEUE_ITEM_ATTEMPT_ENDED_AT_FIELD_NAME]?: Date;
  [PAYOUT_QUEUE_ITEM_ATTEMPT_ERRORS_FIELD_NAME]: {
    [key in PayoutBatchItemCompositeKeySerialized]: string;
  };
};

export type PayoutQueueItemAttemptsT<T> = {
  [attemptId in PayoutQueueItemAttemptId]: T;
};

export type PayoutQueueItemAttempts = PayoutQueueItemAttemptsT<PayoutQueueItemAttempt>;

export type PayoutQueueItem = {
  args: AuthorPayoutArgs;
  left: AuthorPayoutArgs;
  cancelled: boolean;
  initiator: UserId;
  state?: {
    paypal?: {
      batchId: PaypalPayoutBatchId;
    };
  };
  // TODO make it required
  [PAYOUT_QUEUE_ITEM_ATTEMPTS_FIELD_NAME]?: PayoutQueueItemAttempts;
};

export type PayoutQueueStatusResult = z.infer<typeof payoutStatusResultParser>;
export type PayoutQueueQuery = z.input<typeof payoutQueueQueryParser>;
export type PayoutQueueQueryTransformed = z.output<typeof payoutQueueQueryParser>;
export type PayoutQueueResult = z.infer<typeof payoutQueueResultParser>;

export enum PayoutQueueItemAttemptIdBrand {
  _ = '',
}

export type PayoutQueueItemAttemptId = `${typeof PAYOUT_QUEUE_ITEM_ATTEMPT_ID_PREFIX}${string}` &
  PayoutQueueItemAttemptIdBrand;
